@@ -16,11 +16,11 @@ must have a size that is a multiple of nthread  * nfft, where
 nthread is number of threads used, and nfft is the size of the fft. 
 """
 
-__version__ = "0.1.4"
+__version__ = "0.2.0"
 
 from _accelerate_fft_cffi import ffi, lib
 import numpy as np
-from multiprocessing.pool import ThreadPool as Pool
+#from multiprocessing.pool import ThreadPool as Pool
 import threading
 #from multiprocessing import Pool
 from functools import reduce
@@ -53,7 +53,7 @@ def set_nthreads(nthreads):
     FFTSETUP_DATA.clear()
     return current_value
 
-def fft(a, overwrite_x = False, split_in = False, split_out = False):
+def fft(a, n = None, axis = -1, norm = None, overwrite_x = False, split_in = False, split_out = False):
     """Returns a dicrete Fourier transform.
     
     Parameters
@@ -61,6 +61,9 @@ def fft(a, overwrite_x = False, split_in = False, split_out = False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axis : int, optional
+        Axis over which to compute the FFT.  If not given, the last axis is
+        used.
     overwrite_x  : bool
         Specifies whether input data can be overwritten. Whether data is actually
         overwritten depends on the `split_in` and `split_out` arguments.
@@ -77,9 +80,9 @@ def fft(a, overwrite_x = False, split_in = False, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a,overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = +1)
+    return generalized_fft(a, n = n, axis = axis, norm = norm, overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = +1)
 
-def ifft(a, overwrite_x = False, split_in = False, split_out = False):
+def ifft(a, n = None, axis = -1, norm = None, overwrite_x = False, split_in = False, split_out = False):
     """Returns an inverse of the dicrete Fourier transform.
     
     Note that a == ifft(fft(a))/a.shape[-1]
@@ -89,6 +92,9 @@ def ifft(a, overwrite_x = False, split_in = False, split_out = False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axis : int, optional
+        Axis over which to compute the FFT.  If not given, the last axis is
+        used.
     overwrite_x  : bool
         Specifies whether input data can be overwritten. Whether data is actually
         overwritten depends on the `split_in` and `split_out` arguments.
@@ -105,9 +111,9 @@ def ifft(a, overwrite_x = False, split_in = False, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a,overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = -1)
+    return generalized_fft(a, n = n, axis = axis, norm = norm, overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = -1)
     
-def fft2(a, overwrite_x = False, split_in = False, split_out = False):
+def fft2(a, s = None, axes = (-2,-1), norm = None, overwrite_x = False, split_in = False, split_out = False):
     """Returns a dicrete 2D Fourier transform.
         
     Parameters
@@ -115,6 +121,9 @@ def fft2(a, overwrite_x = False, split_in = False, split_out = False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axes : sequence of ints, optional
+        Axes over which to compute the FFT.  If not given, the last two
+        axes are used. Only length 2 axes is supported.
     overwrite_x  : bool
         Specifies whether input data can be overwritten. Whether data is actually
         overwritten depends on the `split_in` and `split_out` arguments.
@@ -131,9 +140,9 @@ def fft2(a, overwrite_x = False, split_in = False, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a,overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = +1, dim = 2)
+    return generalized_fft(a, s = s, axes = axes, norm = norm, overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = +1, dim = 2)
 
-def ifft2(a, overwrite_x = False, split_in = False, split_out = False):
+def ifft2(a, s = None, axes = (-2,-1), norm = None, overwrite_x = False, split_in = False, split_out = False):
     """Returns an inverse of the dicrete 2D Fourier transform.
     
     Note that a == ifft2(fft2(a))/a.shape[-1]/a.shape[-2]
@@ -143,6 +152,9 @@ def ifft2(a, overwrite_x = False, split_in = False, split_out = False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axes : sequence of ints, optional
+        Axes over which to compute the FFT.  If not given, the last two
+        axes are used. Only length 2 axes is supported.
     overwrite_x  : bool
         Specifies whether input data can be overwritten. Whether data is actually
         overwritten depends on the `split_in` and `split_out` arguments.
@@ -159,9 +171,9 @@ def ifft2(a, overwrite_x = False, split_in = False, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a,overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = -1, dim = 2)
+    return generalized_fft(a, s = s, axes = axes, norm = norm, overwrite_x = overwrite_x, split_in = split_in, split_out = split_out, direction = -1, dim = 2)
   
-def rfft(a, split_out = False):
+def rfft(a, n = None, axis = -1, norm = None, split_out = False):
     """Returns a dicrete Fourier transform for real input data.
     
     Note that you should call :func:`unpack`on the computed result to obtain
@@ -171,6 +183,9 @@ def rfft(a, split_out = False):
     ----------
     a : ndarray
         Input real array. 
+    axis : int, optional
+        Axis over which to compute the FFT.  If not given, the last axis is
+        used.
     split_out : bool
         If set, output data is treated as split-complex data. A tuple of two
         real arrays describing the real and imginary parts of the data.
@@ -181,9 +196,9 @@ def rfft(a, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a, split_out = split_out, direction = +1, dim = 1, real_transform = True)
+    return generalized_fft(a, n = n, axis = axis, norm = norm, split_out = split_out, direction = +1, dim = 1, real_transform = True)
 
-def irfft(a, split_in =  False):
+def irfft(a, n = None, axis = -1, norm = None, split_in =  False):
     """Returns an inverse of the dicrete Fourier transform for real input data.
     
     Note that a == irfft(rfft(a))/a.shape[-1]/2 
@@ -193,6 +208,9 @@ def irfft(a, split_in =  False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axis : int, optional
+        Axis over which to compute the FFT.  If not given, the last axis is
+        used.
     split_in : bool
         If set, input data is treated as split-complex data. A tuple of two
         real arrays describing the real and imginary parts of the data.
@@ -202,9 +220,9 @@ def irfft(a, split_in =  False):
     out : ndarray 
         Output float array.
     """
-    return generalized_fft(a, split_in = split_in, direction = -1, dim = 1, real_transform = True)
+    return generalized_fft(a, n = n, axis = axis, norm = norm,  split_in = split_in, direction = -1, dim = 1, real_transform = True)
 
-def rfft2(a, split_out = False):
+def rfft2(a, s = None, axes = (-2,-1), norm = None, split_out = False):
     """Returns a dicrete Fourier transform for 2D real input data.
     
     Note that you should call :func:`unpack2`on the computed result to obtain
@@ -214,6 +232,9 @@ def rfft2(a, split_out = False):
     ----------
     a : ndarray
         Input real array. 
+    axes : sequence of ints, optional
+        Axes over which to compute the FFT.  If not given, the last two
+        axes are used. Only length 2 axes is supported.
     split_out : bool
         If set, output data is treated as split-complex data. A tuple of two
         real arrays describing the real and imginary parts of the data.
@@ -224,9 +245,9 @@ def rfft2(a, split_out = False):
         Output complex array (if split_out = False) or a tuple of float arrays
         (if split_out = True). 
     """
-    return generalized_fft(a, split_out = split_out, direction = +1, dim = 2, real_transform = True)
+    return generalized_fft(a, s = s, axes = axes, norm = norm, split_out = split_out, direction = +1, dim = 2, real_transform = True)
 
-def irfft2(a, split_in = False):
+def irfft2(a, s = None, axes = (-2,-1), norm = None,split_in = False):
     """Returns an inverse of the dicrete Fourier transform for real 2D input data.
     
     Note that a == irfft2(rfft2(a))/a.shape[-1]/a.shape[-2]/2 
@@ -236,6 +257,9 @@ def irfft2(a, split_in = False):
     a : ndarray or (ndarray,ndarray)
         Input complex array (if split_in = False) or a tuple of float arrays
         (if split_in = True). 
+    axes : sequence of ints, optional
+        Axes over which to compute the FFT.  If not given, the last two
+        axes are used. Only length 2 axes is supported.
     split_in : bool
         If set, input data is treated as split-complex data. A tuple of two
         real arrays describing the real and imginary parts of the data.
@@ -245,14 +269,39 @@ def irfft2(a, split_in = False):
     out : ndarray 
         Output float array.
     """
-    return generalized_fft(a, split_in = split_in, direction = -1, dim = 2, real_transform = True)
+    return generalized_fft(a, s = s, axes = axes, norm = norm, split_in = split_in, direction = -1, dim = 2, real_transform = True)
 
-def unpack(a, inplace = False):
+def prepare_input_1d(a,axis = -1,n = None,norm = None):
+    if n is not None:
+        raise ValueError("The 'n' argument is not yet supported!")
+    if norm is not None:
+        raise ValueError("The 'norm' argument is not yet supported!")
+    a = np.asarray(a)
+    return a
+    return np.swapaxes(a,-1,-1)
+
+def prepare_input_2d(a,axes = (-2,-1),s = None,norm = None):
+    return a
+    if s is not None:
+        raise ValueError("The 's' argument is not yet supported!")
+    if norm is not None:
+        raise ValueError("The 'norm' argument is not yet supported!")
+    a = np.asarray(a)
+    try:
+        first, last = [int(i) for i in axes]    
+    except:
+        raise ValueError("Invalid axes argument. Must be a tuple of of length 2")
+    return np.moveaxis(a,(first,last), (-2,-1))
+
+
+def unpack(a, axis = -1, inplace = False):
     """Unpack vDSP rfft data format of shape (...,n/2) into regular
     numpy-like rfft of shape (...,n/2 + 1).
     
     Optionally, this can be done inplace without calculating the last element.
     """
+    a = np.swapaxes(np.asarray(a),axis,-1)
+    
     if inplace == False:
         out = np.empty(a.shape[:-1] + (a.shape[-1]+1,), dtype = a.dtype)
     else:
@@ -262,14 +311,16 @@ def unpack(a, inplace = False):
         out[...,1:-1] = a[...,1:]
     out[...,0] = a[...,0].real
     
-    return out
+    return np.swapaxes(out,-1,axis)
     
-def unpack2(a, inplace = False):
+def unpack2(a, axes = (-2,-1),inplace = False):
     """Unpack vDSP rfft2 data format of shape (..., n, n/2) into regular
     numpy-like rfft2 of shape (..., n, n/2 + 1).
     
     Optionally, this can be done inplace without calculating the last column.
     """
+    a = np.moveaxis(np.asarray(a),axes,(-2,-1))
+    
     n0,n1 = a.shape[-2:] #number of rows
     n0_half = n0//2
 
@@ -301,7 +352,7 @@ def unpack2(a, inplace = False):
         #unpack rest of the data
         out[...,:,1:n1] = a[...,:,1:]
         
-    return out
+    return np.moveaxis(out,(-2,-1),axes)
 
 def create_fftsetup(logn, double = True):
     """Creates and returns a fftsetup of a given logsize and precision.
@@ -316,7 +367,7 @@ def create_fftsetup(logn, double = True):
     double : bool
         Indicates whether we are working with double precision (default) or not.
     """
-    #this function must be thread-safe, so we acquire alock here
+    #this function must be thread-safe, so we acquire a lock here
     with lock:
         if double == True:
             return fftsetupD.create(logn)
@@ -470,23 +521,26 @@ def _optimal_workers(size, nthreads):
     else:
         return _optimal_workers(size, nthreads-1)
 
-
 def _optimal_flattened_shape(original_shape, dim = 1):
     """Finds a shape for a flattened array, so that we can then iterate over it"""
     nthreads = fft_config["nthreads"]
     if len(original_shape) == 1:
-        #has to be 2D data
-        original_shape = (1,) + original_shape
+        #Even for 1D transforms data has to be 3D data. 
+        # algorithm iterates over the first axis, transform is done on 2D data.
+        return (1,1) + original_shape
     
-    newshape = reduce((lambda x,y: x*y), original_shape[:-2] or [1])
-    if fft_config["nthreads"] > 1:
-        n = _optimal_workers(newshape,nthreads)
-        newshape = (n,newshape//n,) + original_shape[-2:]
+    elif len(original_shape) == 2:
+        if dim == 1:
+            # we need to reduce the shape
+            newshape = reduce((lambda x,y: x*y), original_shape[:-1])
+            n = _optimal_workers(newshape,nthreads)
+            return (n,newshape//n) + original_shape[:-1]
+        else:
+            return (1,) + original_shape
     else:
-        newshape = (newshape,) + original_shape[-2:]
-    return newshape
+        newshape = reduce((lambda x,y: x*y), original_shape[:-2])
+        return (newshape,) + original_shape[-2:]
 
-        
       
 def _create_split_complex_pointer(array_real, array_imag, double = False):
     """Creates a split-complex data pointer from two real numpy arrays"""
@@ -527,16 +581,52 @@ def _get_out_shape(in_shape, dim = 1, direction = +1, real_transform = False):
     else:
         return in_shape
      
-def _init_setup_and_arrays(a, overwrite_x, dim = 1, split_in = False, split_out = False, direction = +1, real_transform = False):
+def _init_setup_and_arrays(a, n = None, s = None, axis = -1, axes = (-2,-1), norm = None, overwrite_x = False, dim = 1, split_in = False, split_out = False, direction = +1, real_transform = False):
     """checks input parameters and creates a valid fft setup and input/output arrays"""
-    # make an array and read dtype, and shape
-    a = (np.asarray(a[0]),np.asarray(a[1])) if split_in else np.asarray(a)
-    dtype = a[0].dtype if split_in else a.dtype
-    shape = a[0].shape if split_in else a.shape
     
+    if n is not None:
+        raise ValueError("The 'n' argument is not yet supported!")
+    if s is not None:
+        raise ValueError("The 's' argument is not yet supported!")
+    if norm is not None:
+        raise ValueError("The 'norm' argument is not yet supported!")
+    
+    # make an array and read dtype, and shape
+    if split_in:
+        a0,a1 = np.asarray(a[0]),np.asarray(a[1])
+        if dim == 1:
+            a0 = np.swapaxes(a0,axis,-1)
+            a1 = np.swapaxes(a1,axis,-1)
+        else:
+            a0 = np.moveaxis(a0,axes,(-2,-1))
+            a1 = np.moveaxis(a1,axes,(-2,-1))            
+        dtype = a0.dtype
+        shape = a0.shape 
+        new_shape = _optimal_flattened_shape(a0.shape, dim = dim)
+        a0, a1 = (a0.reshape(new_shape), a1.reshape(new_shape))
+        if not a0.data.c_contiguous:
+            a0 = a0.copy()
+        if not a1.data.c_contiguous:
+            a1 = a1.copy()
+        a =  (a0, a1)
+    else:
+        a = np.asarray(a)
+        if dim == 1:
+            a = np.swapaxes(a,axis,-1)
+        else:
+            a = np.moveaxis(a,axes,(-2,-1))  
+
+        dtype = a.dtype
+        shape = a.shape 
+        new_shape = _optimal_flattened_shape(a.shape, dim = dim)
+        a = a.reshape(new_shape)  
+        if not a.data.c_contiguous:
+            a = a.copy()
+
+
     # determine the allowed type and shapes of the input and output arrays
     in_dtype, out_dtype = _get_in_out_dtype(dtype, split_in, split_out, direction, real_transform)
-    out_shape = _get_out_shape(shape, dim, direction, real_transform)
+    out_shape = _get_out_shape(new_shape, dim, direction, real_transform)
 
     # make sure it is right type
     a = tuple((np.asarray(d,in_dtype) for d in a)) if split_in else np.asarray(a,in_dtype)  
@@ -550,7 +640,7 @@ def _init_setup_and_arrays(a, overwrite_x, dim = 1, split_in = False, split_out
         out = (np.empty(out_shape,out_dtype), np.empty(out_shape,out_dtype)) if split_out else np.empty(out_shape,out_dtype) 
     #determine if we need to use double precision    
     double = dtype in ("float64","complex128") 
-    key = (shape[-2:], out_shape[-2:],split_in, split_out, dim, double,direction,real_transform)
+    key = (new_shape[-2:], out_shape[-2:],split_in, split_out, dim, double,direction,real_transform)
     try:
         #if setup already exists, just take one.
         setup = FFTSETUP_DATA[key] 
@@ -559,28 +649,101 @@ def _init_setup_and_arrays(a, overwrite_x, dim = 1, split_in = False, split_out
         setup = [FFTSetupData(*key) for i in range(fft_config["nthreads"])]    
         FFTSETUP_DATA[key] = setup
     
-    return setup, a, out
+    return shape, setup, a, out
 
 #-----------------
 # Worker functions
 #-----------------
 
+
+POOL = {}
+
+from threading import Thread
+from queue import Queue
+
+class PoolWorker():
+    """Mimics the object returned by ThreadPool.apply_async method."""
+    def __init__(self, queue):
+        self.queue = queue
+        
+    def get(self):
+        return self.queue.get()
+
+class Pool:
+    """A multiprocessing.pool.ThreadPool -like object.
+    
+    Implements only necessary part of ThreadPool API.
+    """
+    def __init__(self,nthreads):
+        def worker(i, inputs, results):
+            #print("Thread ", i, "started")
+            while True:
+                data = inputs.get()
+                if data is None:
+                    results.put(None)
+                    break
+                else:
+                    func, args, kwargs = data
+                    out = func(*args,**kwargs)
+                    results.put((i,out))
+            #print("Thread ", i, "stopped")
+            
+            
+        self.nthreads = nthreads
+        self.results = [Queue() for i in range(nthreads)]
+        self.inputs = [Queue() for i in range(nthreads)]
+        self.threads = [Thread(target = worker, args = (i,self.inputs[i],self.results[i]), daemon = True) for i in range(nthreads)]
+        [t.start() for t in self.threads]
+        self.nruns = 0
+        
+    def apply_async(self,func, args = (), kwds = {}):
+        index = self.nruns % self.nthreads
+        self.inputs[index].put((func,args,kwds))
+        self.nruns += 1
+        return PoolWorker(self.results[index])
+              
+    def close(self):
+        for q in self.inputs:
+            q.put(None)
+        for q in self.results:
+            while q.get(timeout = 1) != None:
+                pass
+        for t in self.threads:
+            t.join()
+                 
+    def __del__(self):
+        self.close()
+
+def clear_pool():
+    """Clears thread pool. Deletes all Pool objects, which terminates all
+    running background threads."""
+    
+    POOL.clear()
+
 def _sequential_call(fftfunc,setup, *args,**kwargs):
     # a simple sequential runner 
     [fftfunc(setup,*arg,**kwargs) for arg in zip(*args)] 
-
+   
 def _calculate_fft(fftfunc,*args,**kwds):
     """Runs fft function with given arguments (optionally in parallel 
     using a ThreadPool)"""
     nthreads = fft_config["nthreads"]
 
     if nthreads > 1:
-        pool = Pool(nthreads)
-        workers = [pool.apply_async(_sequential_call, args = (fftfunc,) + arg, kwds = kwds) for arg in zip(*args)] 
+        try:
+            pool = POOL[nthreads]
+        except KeyError:
+            pool = Pool(nthreads)
+            POOL[nthreads] = pool
+        #pool = Pool(nthreads)
+        setup = args[0]
+        args = args[1:]
+        workers = [pool.apply_async(fftfunc, args = (setup[i%nthreads],) + arg, kwds = kwds) for i,arg in enumerate(zip(*args))] 
         _ = [w.get() for w in workers]
-        pool.close()
+        #pool.close()
     else:
         setup = args[0]
+        #setup is a list. For single-threaded run, take the first and only element of the list.
         args = (setup[0],) + args[1:]
         _sequential_call(fftfunc,*args, **kwds)
 
@@ -635,7 +798,7 @@ def _fft(setup, a, out, n = 1, dim = 1, direction = +1, real_transform = False):
     
 def _ffts(setup, a, outr, outi, n = 1, dim = 1, direction = +1, real_transform = False):
     """Same as _fft, but it leaves the data in split-complex format"""
-    
+ 
     #initialize
     _a = ffi.from_buffer(a) 
     _pa = ffi.cast(setup.cast_name,_a)
@@ -673,6 +836,7 @@ def _sffti(setup, ar, ai, out, n = 1, dim = 1, direction = +1, real_transform =
 
 def _sffto(setup, ar, ai, out, n = 1, dim = 1, direction = +1, real_transform = False):
     """1D or 2D fft transform, real or complex for split-complex input data - out of place"""
+
     _ps = _create_split_complex_pointer(ar, ai, double = setup.double)
     _out = ffi.from_buffer(out) 
     _pout = ffi.cast(setup.cast_name,_out)   
@@ -688,6 +852,7 @@ def _sffto(setup, ar, ai, out, n = 1, dim = 1, direction = +1, real_transform =
 
 def _sfftis(setup, ar,ai, n = 1, dim = 1, direction = +1):
     """Same as _ffts (complex transform only), but with input data as split complex data. Inplace transform"""
+
     _ps = _create_split_complex_pointer(ar, ai, double = setup.double)
     _func = _get_vDSP_fft_inplace(double = setup.double, dim = dim)
     if dim == 1:
@@ -697,6 +862,7 @@ def _sfftis(setup, ar,ai, n = 1, dim = 1, direction = +1):
 
 def _sfftos(setup, ar,ai, outr, outi, n = 1, dim = 1, direction = +1):
     """out-of-place version of _sfftis"""
+
     _pins = _create_split_complex_pointer(ar, ai, double = setup.double)
     _pouts = _create_split_complex_pointer(outr, outi, double = setup.double)
     _func = _get_vDSP_fft_outplace(double = setup.double, dim = dim)
@@ -705,51 +871,56 @@ def _sfftos(setup, ar,ai, outr, outi, n = 1, dim = 1, direction = +1):
     else:
         _func(setup.pointer, _pins, 1,0, _pouts, 1,0,setup.size[0], setup.size[1],direction)
 
-def generalized_fft(a,dim = 1, overwrite_x = False, split_in = False, split_out = False, direction = +1, real_transform = False):
-    setup, a, out = _init_setup_and_arrays(a, overwrite_x, dim = dim, split_in = split_in, split_out = split_out, direction = direction, real_transform = real_transform)
+def generalized_fft(a,n = None, s = None, axis = -1, axes = (-2,-1), norm = None, dim = 1, overwrite_x = False, split_in = False, split_out = False, direction = +1, real_transform = False):
+    original_shape, setup, a, out = _init_setup_and_arrays(a, n = n, s = s, axis = axis, axes = axes, overwrite_x = overwrite_x, dim = dim, split_in = split_in, split_out = split_out, direction = direction, real_transform = real_transform)
     
     if split_in == True:
         #a must be a tuple of length twp for the two arrays
         a_real, a_imag = a
-        # reshape data, so that we can compute fft sequentially.
-        shape = _optimal_flattened_shape(a_real.shape, dim = dim)
-        _a_real, _a_imag = a_real.reshape(shape), a_imag.reshape(shape)
-        
-        n = shape[-2]
+
+        n = a_real.shape[-2]
        
         if split_out == True:
             if overwrite_x == False:
                 #out of place transform
                 out_real, out_imag = out
-                shape = _optimal_flattened_shape(out_real.shape,  dim = dim)
-                _out_real, _out_imag =  out_real.reshape(shape), out_imag.reshape(shape)
-                _calculate_fft(_sfftos , setup, _a_real,_a_imag,_out_real,_out_imag, n = n, dim = dim, direction = direction)
+                _calculate_fft(_sfftos , setup, a_real,a_imag,out_real,out_imag, n = n, dim = dim, direction = direction)
+
             else:
                 #inplace transform
-                _calculate_fft(_sfftis, setup, _a_real,_a_imag, n = n, dim = dim, direction = direction)
+                _calculate_fft(_sfftis, setup, a_real,a_imag, n = n, dim = dim, direction = direction)
         else:
-            shape = _optimal_flattened_shape(out.shape,  dim = dim)
-            _out = out.reshape(shape)
             if overwrite_x == True:
-                _calculate_fft(_sffti, setup, _a_real,_a_imag,_out, n = n, dim = dim, direction = direction, real_transform = real_transform)
+                _calculate_fft(_sffti, setup, a_real,a_imag,out, n = n, dim = dim, direction = direction, real_transform = real_transform)
             else:
-                _calculate_fft(_sffto, setup, _a_real,_a_imag,_out, n = n, dim = dim, direction = direction, real_transform = real_transform)
+                _calculate_fft(_sffto, setup, a_real,a_imag,out, n = n, dim = dim, direction = direction, real_transform = real_transform)
 
     else:
-        shape = _optimal_flattened_shape(a.shape, dim = dim)
-        n = shape[-2]
-        _a = a.reshape(shape)
-            
+
+        n = a.shape[-2]
+                  
         if split_out == False:
-            shape = _optimal_flattened_shape(out.shape, dim = dim)
-            _out =  out.reshape(shape)
-            _calculate_fft(_fft, setup, _a,_out, n = n, dim = dim, direction = direction, real_transform = real_transform)
+            _calculate_fft(_fft, setup, a,out, n = n, dim = dim, direction = direction, real_transform = real_transform)
         else:
-            shape = _optimal_flattened_shape(out[0].shape,  dim = dim)
-            _out_real, _out_imag =  out[0].reshape(shape), out[1].reshape(shape)
-            _calculate_fft(_ffts, setup, _a,_out_real,_out_imag, n = n, dim = dim, direction = direction, real_transform = real_transform)
-    
-    return out   
+            out_real, out_imag =  out
+            _calculate_fft(_ffts, setup, a,out_real,out_imag, n = n, dim = dim, direction = direction, real_transform = real_transform)
+    if split_out:
+        out_shape = original_shape[:-1] + out[0].shape[-1:]
+        x,y =  out[0].reshape(out_shape), out[1].reshape(out_shape)
+        if dim == 2:
+            return np.moveaxis(x,(-2,-1),axes),np.moveaxis(y,(-2,-1),axes)
+        else:
+            return np.swapaxes(x,-1,axis),np.swapaxes(y,-1,axis)
+        
+    else:
+        out_shape = original_shape[:-1] + out.shape[-1:]
+        z = out.reshape(out_shape)  
+        if dim == 2:
+            return np.moveaxis(z,(-2,-1),axes)
+        else:
+            return np.swapaxes(z,-1,axis)      
+
+
 
 
 
